/*
 * reg230.c
 *
 * Created: 2016-08-01 09:31:59
 *  Author: dsosnowski
 */ 

//---------------------------------------------------
#define F_CPU					8000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "wdog.h"
#include "harddef.h"
#include "lcd.h"
#include "ee_mem.h"

#define MAIN_DELAY				1

#define ACTION_IMP_THR			(200 / MAIN_DELAY)
#define ACTION_IDLE				0
#define ACTION_IMP_R			1
#define ACTION_IMP_L			2
#define ACTION_SW				3
#define ACTION_IMPSW			4
#define ACTION_HOLD				16
#define BUTTON_THRESHOLD		(2000 / MAIN_DELAY)
#define BUTTON_REPEAT			(20 / MAIN_DELAY)

#define ACTION_MULTI_TIM		(200 / MAIN_DELAY)
#define ACTION_PASS_COUNTER		12
#define ACTION_MULTI			16

#define VOLUME_MAX				100
#define VOLUME_MIN				0
#define VOLUME_MULTISTEP		10

#define BLINK_PERIOD			500
#define RANGE_REG				OCR1A
#define POWER_REG				OCR1B

#define POWER_ENABLE			1
#define POWER_OFF				0
#define PHASE_MODE				0
#define GROUP_MODE				1
#define GROUP10_MODE			2			


void HardInit(void);
void UiScan(void);
char UiActionMulti(char button);


struct {
	char action;
	char volume;
	char refresh;
	char newset;
	char blink;
	int16_t i;
	uint16_t startup;
} ui;

struct {
	char enable;
	char mode;
	uint16_t reg;
	volatile char group;
	char group_reg;
	char group_range;
} power;



//---------------------------------------------------
int main(void)
{
	int16_t v = 0;
	
	HardInit();
	WdtSet(7);
	LCDInit();                   
	LCDMsg(0,0," REGULAT");
	LCDMsg(1,0,"OR MOCY ");
	ui.startup = (1500/MAIN_DELAY);
	ui.i = -1000;
	
	ui.volume = EEread(5);
	power.mode = EEread(6);
	power.enable = EEread(7);
	if (ui.volume > 100) ui.volume = 0;
	if (power.mode > 2) power.mode = 0;
	if (power.enable > 1) power.enable = 0;
	
	sei();
	
	//---------------------------------------------------
    while(1)
    {
	    //---------------------------------------------------
        UiScan();
		if (ui.action > 0) {							
			if (ui.action == ACTION_SW) {
				power.enable++;
				if (power.enable > POWER_ENABLE) power.enable = 0;
				ui.newset = 1;
			}
			if (ui.action == (ACTION_SW + ACTION_HOLD)) {
				cli();
				EEwrite(5, ui.volume);
				EEwrite(6, power.mode);
				EEwrite(7, power.enable);
				sei();
				LCDMsg(0,0,"ZAPISANO");
				_delay_ms(1000);
			}
			if (ui.action == ACTION_IMPSW) {
				if (power.enable == POWER_OFF) {
					power.mode++;
					if (power.mode > GROUP10_MODE) power.mode = 0;
				}
			}
			
			if (ui.action == (ACTION_IMP_R + ACTION_MULTI)){
				ui.volume += (VOLUME_MULTISTEP - 1);
				ui.action = ACTION_IMP_R;
			}
			if (ui.action == ACTION_IMP_R) {
				ui.volume++;
				if (power.mode == GROUP10_MODE) ui.volume += 9;
				if (ui.volume > VOLUME_MAX) ui.volume = VOLUME_MAX;
				ui.newset = 1;
			}
			
			if (ui.action == (ACTION_IMP_L + ACTION_MULTI)){
				ui.volume -= (VOLUME_MULTISTEP - 1);
				ui.action = ACTION_IMP_L;
			}
			if (ui.action == ACTION_IMP_L) {
				ui.volume--;
				if (power.mode == GROUP10_MODE) ui.volume -= 9;
				if (ui.volume > VOLUME_MAX) ui.volume = VOLUME_MIN;				
				ui.newset = 1;
			}
			
			if (power.mode == GROUP10_MODE) {
				ui.volume -= (ui.volume % 10);
			}
			
			ui.refresh = 1;
		}
		
		//---------------------------------------------------
		if (ui.startup) {
			ui.startup--;
			if (ui.startup == 0) {
				ui.refresh = 1;
				ui.newset = 1;
			}
		}
		
		//---------------------------------------------------
		if (power.enable == POWER_ENABLE){
			ui.i++;
			if (ui.i >= BLINK_PERIOD) {
				ui.blink++;
				ui.refresh = 1;
				ui.i = 0;
			}
		} else {
			ui.i = 0;
		}
		
		//---------------------------------------------------
		if (ui.refresh) {
			LCDMsg(0,0,"MOC:");
			LCDValue3(ui.volume);
			LCDText("%");
						
			LCDLocate(1,0);
			if (power.enable == POWER_ENABLE) {
				if ((ui.blink & 1) == 0){
					LCDText(" AKTYWNY");	
				} else {
					LCDText("        ");					
				}
			} else {
				if (power.mode == PHASE_MODE){
					LCDText(" Fazowa ");					
				}
				if (power.mode == GROUP_MODE){
					LCDText(" Grupowa");
				}
				if (power.mode == GROUP10_MODE){
					LCDText(" Grup.10");
				}
			}
			ui.refresh = 0;	
		}		
		
		//---------------------------------------------------
		if (ui.newset > 0){
			v = 0xFFFF;
			if (power.enable == POWER_ENABLE){
				if (ui.volume > 0){
					if (power.mode == PHASE_MODE) {
						power.group_reg = 0xFF;
						if (ui.volume >= 100) ui.volume = 100;
						v = (100 - ui.volume);
						v = (v * 100);
						if (v == 0)	 v = 10;
					} else {
						v = 10;
						if (power.mode == GROUP_MODE) {
							power.group_range = 200;					
							power.group_reg = (ui.volume * 2);
						}
						if (power.mode == GROUP10_MODE) {
							power.group_range = 20;							
							power.group_reg = (ui.volume / 5);
						}
					}				
				}
			}
			ui.newset = 0;
			power.reg = v;
		}
		
		//---------------------------------------------------
		_delay_ms(MAIN_DELAY);
		WdtReset();
    }
}


//---------------------------------------------------
void HardInit(void){
	LCD_PORT = 0XFF;
	LCD_DDR = 0;
	
	IMP_PORT |= (0<<IMP_A | 0<<IMP_B | 0<<IMP_SW | 0<<SW1_PIN);
	
	PHASE_PORT |= (1<<PHASE_IN | 1<<PHASE_OUT);
	PHASE_DDR |= (1<<PHASE_OUT);
	
	MCUCR = (1<<ISC01 | 0<<ISC00);
	GIMSK = (1<<INT0);
	
	RANGE_REG = 11000;		
	POWER_REG = 0xFFFF;
	TCCR1A = (0<<WGM11 | 0<<WGM10);
	TCCR1B = (0<<WGM13 | 1<<WGM12 | 0);
	TIMSK = (1<<OCIE1A | 1<<OCIE1B);
}

//---------------------------------------------------
ISR(INT0_vect){
	GTCCR = (1<<PSR10);
	TCNT1 = 0;
	PHASE_OFF;
	POWER_REG = power.reg;
	power.group++;
	if (power.group >= power.group_range) power.group = 0;
	TCCR1B = (0<<WGM13 | 1<<WGM12 | 2);
}

//---------------------------------------------------
ISR(TIMER1_COMPB_vect){
	if (power.group < power.group_reg)
		PHASE_ON;
}
//---------------------------------------------------
ISR(TIMER1_COMPA_vect){
	TCNT1 = 0;
	PHASE_OFF;
	TCCR1B = (0<<WGM13 | 1<<WGM12 | 2);
}

//-------------------------------------------------
void UiScan(void) {
	static char impprev;
	static char impmem;
	static uint16_t impdelay;
	
	char button;
	static uint16_t hold;
	static char prev;
	
	char pin;
	char action;
	
	action = 0;
	pin = IMP_IN;
	
	
	//--------------------------------------------------------
	if (impdelay < 0xFFFF) impdelay++;

	if ((pin & (1<<IMP_A)) > 0) {
		impprev = 0;
	} else {
		if (impprev == 0) {
			if (impdelay > ACTION_IMP_THR) {
				if ((pin & (1<<IMP_B)) > 0) {
					action = ACTION_IMP_R;
					impmem = ACTION_IMP_R;
				} else {
					action = ACTION_IMP_L;
					impmem = ACTION_IMP_L;
				}
			} else {
				action = impmem;
			}
			impdelay = 0;
		}
		impprev = 1;
	}
	

	//--------------------------------------------------------
	button = 0;
	if ((pin & (1<<IMP_SW)) > 0) button = ACTION_IMPSW;
	if ((pin & (1<<SW1_PIN)) > 0) button = ACTION_SW;
	
	if (button == 0) {
		if ((hold > 5) && (hold < BUTTON_THRESHOLD)) {
			action = prev;
		}
		prev = 0;
		hold = 0;
	}
	else
	{
		if (hold <= BUTTON_THRESHOLD) hold++;
		if (button != prev) hold = 0;
		prev = button;
		button = 0;
		
		if (hold == BUTTON_THRESHOLD) {
			action = (prev + ACTION_HOLD);
			//if ((prev == ACTION_SW1) || (prev == ACTION_SW2)) {
			//	hold -= BUTTON_REPEAT;
			//}
		}
	}
	
	ui.action = (action + UiActionMulti(action));
	//system.action = action;
}

//-------------------------------------------------
char UiActionMulti(char button) {
	static char i;
	static char prev;
	static char pass_button;
	char result = 0;
	
	if (i <= ACTION_MULTI_TIM) i++;
	if (i == ACTION_MULTI_TIM) {
		prev = 0;
		pass_button = 0;
	}

	if (button > ACTION_IDLE) {
		i = 0;

		if (button == prev) {
			if (pass_button < ACTION_PASS_COUNTER) {
				pass_button++;
			}
			} else {
			pass_button = 0;
		}
		prev = button;

		if (pass_button >= ACTION_PASS_COUNTER) {
			result = ACTION_MULTI;
		}
	}
	return result;
}

